@@ -26,6 +26,7 @@
 #include <oblibs/stack.h>
 #include <oblibs/directory.h>
 #include <oblibs/io.h>
+#include <oblibs/lexer.h>
 
 #include <skalibs/sgetopt.h>
 #include <skalibs/tai.h>
@@ -37,7 +38,6 @@
 #include <66/service.h>
 #include <66/constants.h>
 #include <66/utils.h>
-#include <66/enum.h>
 
 #include <s6/fdholder.h>
 
@@ -352,30 +352,30 @@ static void io_setup_stdin(resolve_service_t *res)
 
     switch(res->io.fdin.type) {
 
-        case IO_TYPE_TTY:
+        case E_PARSER_IO_TYPE_TTY:
             io_open_terminal_ncontrol(res, 0, res->sa.s + res->io.fdin.destination, O_RDWR | O_NOCTTY) ;
             break ;
 
-        case IO_TYPE_NULL:
+        case E_PARSER_IO_TYPE_NULL:
             io_open_destination(0, res->sa.s +  res->io.fdin.destination, O_RDONLY | O_NOCTTY, 0, 0) ;
             break ;
 
-        case IO_TYPE_CLOSE:
+        case E_PARSER_IO_TYPE_CLOSE:
             fd_close(0) ;
             break ;
 
-        case IO_TYPE_S6LOG:
-            if (res->type == TYPE_CLASSIC && res->islog)
+        case E_PARSER_IO_TYPE_S6LOG:
+            if (res->type == E_PARSER_TYPE_CLASSIC && res->islog)
                 io_fdholder_retrieve(res, 0, res->sa.s + res->name, 0) ;
             break ;
 
-        case IO_TYPE_PARENT:
+        case E_PARSER_IO_TYPE_PARENT:
             break ;
 
-        case IO_TYPE_CONSOLE:
-        case IO_TYPE_FILE:
-        case IO_TYPE_SYSLOG:
-        case IO_TYPE_INHERIT:
+        case E_PARSER_IO_TYPE_CONSOLE:
+        case E_PARSER_IO_TYPE_FILE:
+        case E_PARSER_IO_TYPE_SYSLOG:
+        case E_PARSER_IO_TYPE_INHERIT:
             break ;
         default:
             log_warn("unknown StdIn type -- applying default") ;
@@ -389,17 +389,17 @@ static void io_setup_stdout(resolve_service_t *res)
 
     switch(res->io.fdout.type) {
 
-        case IO_TYPE_CONSOLE:
+        case E_PARSER_IO_TYPE_CONSOLE:
             io_open_active_console(1, res) ;
             break ;
 
-        case IO_TYPE_S6LOG:
+        case E_PARSER_IO_TYPE_S6LOG:
 
-            if (res->type == TYPE_CLASSIC && !res->islog) {
+            if (res->type == E_PARSER_TYPE_CLASSIC && !res->islog) {
 
                 io_fdholder_retrieve(res, 1, res->sa.s + res->logger.name, 1) ;
 
-            } else if (res->type == TYPE_ONESHOT) {
+            } else if (res->type == E_PARSER_TYPE_ONESHOT) {
 
                 _alloc_stk_(stk, strlen(res->sa.s + res->io.fdout.destination) + SS_TREE_CURRENT_LEN + 2) ;
                 auto_strings(stk.s, res->sa.s + res->io.fdout.destination, "/", SS_TREE_CURRENT) ;
@@ -409,30 +409,30 @@ static void io_setup_stdout(resolve_service_t *res)
             }
             break ;
 
-        case IO_TYPE_TTY:
+        case E_PARSER_IO_TYPE_TTY:
             io_open_terminal(res, 1, res->sa.s + res->io.fdout.destination, O_WRONLY | O_NOCTTY) ;
             break ;
 
-        case IO_TYPE_NULL:
+        case E_PARSER_IO_TYPE_NULL:
             io_open_destination(1, res->sa.s + res->io.fdout.destination, O_WRONLY, 0, 0) ;
             break ;
 
-        case IO_TYPE_FILE:
+        case E_PARSER_IO_TYPE_FILE:
             io_open_file(res, 1, res->sa.s + res->io.fdout.destination) ;
             break ;
 
-        case IO_TYPE_SYSLOG:
+        case E_PARSER_IO_TYPE_SYSLOG:
             io_open_syslog(1) ;
             break ;
 
-        case IO_TYPE_CLOSE:
+        case E_PARSER_IO_TYPE_CLOSE:
             fd_close(1) ;
             break ;
 
-        case IO_TYPE_PARENT:
+        case E_PARSER_IO_TYPE_PARENT:
             break ;
 
-        case IO_TYPE_INHERIT:
+        case E_PARSER_IO_TYPE_INHERIT:
             if (fd_copy(1, 0) < 0)
                 log_dieusys(LOG_EXIT_SYS, "copy stdout to stderr") ;
             break ;
@@ -449,36 +449,36 @@ static void io_setup_stderr(resolve_service_t *res)
 
     switch(res->io.fderr.type) {
 
-        case IO_TYPE_CONSOLE:
+        case E_PARSER_IO_TYPE_CONSOLE:
             io_open_active_console(2, res) ;
             break ;
 
-        case IO_TYPE_TTY:
+        case E_PARSER_IO_TYPE_TTY:
             io_open_terminal(res, 2, res->sa.s + res->io.fderr.destination, O_WRONLY | O_NOCTTY) ;
             break ;
 
-        case IO_TYPE_NULL:
+        case E_PARSER_IO_TYPE_NULL:
             io_open_destination(2, res->sa.s + res->io.fdout.destination, O_WRONLY, 0, 0) ;
             break ;
 
-        case IO_TYPE_FILE:
+        case E_PARSER_IO_TYPE_FILE:
             io_open_file(res, 2, res->sa.s + res->io.fderr.destination) ;
             break ;
 
-        case IO_TYPE_SYSLOG:
+        case E_PARSER_IO_TYPE_SYSLOG:
             io_open_syslog(2) ;
             break ;
 
-        case IO_TYPE_CLOSE:
+        case E_PARSER_IO_TYPE_CLOSE:
             fd_close(2) ;
             break ;
 
-        case IO_TYPE_INHERIT:
+        case E_PARSER_IO_TYPE_INHERIT:
             if (fd_copy(2, 1) < 0)
                 log_dieusys(LOG_EXIT_SYS, "copy stderr to stdout") ;
             break ;
 
-        case IO_TYPE_PARENT:
+        case E_PARSER_IO_TYPE_PARENT:
             break ;
 
         default:
@@ -487,7 +487,7 @@ static void io_setup_stderr(resolve_service_t *res)
     }
 }
 
-static void execute_environment(stralloc *e, char const **nenvp, char const *const *env, exlsn_t *info, resolve_service_t *res)
+static void execute_environment(char const **nenvp, char const *const *env, stralloc *eram, exlsn_t *info, resolve_service_t *res)
 {
     log_flow() ;
 
@@ -500,17 +500,32 @@ static void execute_environment(stralloc *e, char const **nenvp, char const *con
             !stack_close(&path))
                 log_die_nomem("stack") ;
 
-        if (!environ_merge_dir(e, path.s))
+        if (!environ_merge_dir(eram, path.s))
             log_dieusys(LOG_EXIT_SYS, "merge environment directory: ", path.s) ;
 
-        if (!environ_substitute(e, info))
+        if (res->environ.nimportfile) {
+
+            _alloc_stk_(stk, strlen(res->sa.s + res->environ.importfile)) ;
+            size_t pos = 0 ;
+
+            if (!stack_string_clean(&stk, res->sa.s + res->environ.importfile))
+                log_dieusys(LOG_EXIT_ZERO, "clean string") ;
+
+            FOREACH_STK(&stk, pos) {
+
+                if (!environ_merge_file(eram, stk.s + pos))
+                    log_dieusys(LOG_EXIT_ZERO, "merge environment file: ", stk.s + pos) ;
+            }
+        }
+
+        if (!environ_substitute(eram, info))
             log_dieusys(LOG_EXIT_SYS, "substitue environment variables") ;
 
-        if (!environ_clean_unexport(e))
+        if (!environ_clean_unexport(eram))
             log_dieusys(LOG_EXIT_SYS, "remove exclamation mark from environment") ;
     }
 
-    if (!environ_create_environ(nenvp, env, e))
+    if (!environ_create_environ(nenvp, env, eram))
         log_dieusys(LOG_EXIT_SYS, "create environment") ;
 }
 
@@ -522,7 +537,7 @@ static void execute_script(const char *runuser, resolve_service_t *res, exlsn_t
     _alloc_sa_(sa) ;
     uid_t owner = getuid() ;
     uint32_t want = (action == EXECUTE_START) ? res->execute.run.build : res->execute.finish.build ;
-    short build = !strcmp(res->sa.s + want, "custom") ? BUILD_CUSTOM : BUILD_AUTO ;
+    short build = !strcmp(res->sa.s + want, "custom") ? E_PARSER_BUILD_CUSTOM : E_PARSER_BUILD_AUTO ;
     char *script = res->sa.s + (action == EXECUTE_START ? res->execute.run.run_user : res->execute.finish.run_user) ;
     size_t scriptlen = strlen(script) ;
 
@@ -624,7 +639,7 @@ int main(int argc, char const *const *argv, char const *const *envp)
 
     char const *service = 0 ;
     _alloc_stk_(base, SS_MAX_PATH + 1) ;
-    _alloc_sa_(env) ;
+    _alloc_sa_(eram) ; // envrionment in memory
     char const *nenvp[MAXENV + 1] ;
     char *run = 0 ;
     char *runuser = 0 ;
@@ -702,7 +717,7 @@ int main(int argc, char const *const *argv, char const *const *envp)
             log_dieusys(LOG_EXIT_SYS, "find script: ", brun, " -- please make a bug report") ;
     }
 
-    execute_environment(&env, nenvp, envp, &info, &res) ;
+    execute_environment(nenvp, envp, &eram, &info, &res) ;
 
     execute_script(brunuser, &res, &info) ;
 
