@@ -17,10 +17,13 @@
 
 #include <66/config.h>
 #include <66/constants.h>
+#include <66/enum_parser.h>
+#include <66/enum_service.h>
+#include <66/enum_struct.h>
+#include <66/enum_tree.h>
 #include <66/enum.h>
 #include <66/environ.h>
 #include <66/graph.h>
-#include <66/hash.h>
 #include <66/hpr.h>
 #include <66/info.h>
 #include <66/instance.h>
