@@ -16,5 +16,10 @@
 #define SS_MIGRATE_H
 
 #include <66/migrate_0721.h>
+#include <66/migrate_0802.h>
+
+extern void migrate_create_snap(ssexec_t *info, const char *version) ;
+extern void migrate_0721(ssexec_t *info) ;
+extern void migrate_0802() ;
 
 #endif
