@@ -20,6 +20,7 @@
 #include <wchar.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <errno.h>
 
 #include <oblibs/sastr.h>
 #include <oblibs/log.h>
@@ -27,7 +28,6 @@
 #include <oblibs/string.h>
 #include <oblibs/files.h>
 #include <oblibs/directory.h>
-#include <oblibs/graph.h>
 #include <oblibs/environ.h>
 #include <oblibs/sastr.h>
 
@@ -43,7 +43,7 @@
 #include <66/info.h>
 #include <66/constants.h>
 #include <66/tree.h>
-#include <66/enum.h>
+#include <66/enum_parser.h>
 #include <66/resolve.h>
 #include <66/environ.h>
 #include <66/state.h>
@@ -82,6 +82,7 @@ static void info_display_start(char const *field, resolve_service_t *res) ;
 static void info_display_stop(char const *field, resolve_service_t *res) ;
 static void info_display_envat(char const *field, resolve_service_t *res) ;
 static void info_display_envfile(char const *field, resolve_service_t *res) ;
+static void info_display_importfile(char const *field, resolve_service_t *res) ;
 static void info_display_stdin(char const *field, resolve_service_t *res) ;
 static void info_display_stdout(char const *field, resolve_service_t *res) ;
 static void info_display_stderr(char const *field, resolve_service_t *res) ;
@@ -114,15 +115,16 @@ static info_opts_map_t const opts_sv_table[] =
     { .str = "stop", .svfunc = &info_display_stop, .id = 17 },
     { .str = "envat", .svfunc = &info_display_envat, .id = 18 },
     { .str = "envfile", .svfunc = &info_display_envfile, .id = 19 },
-    { .str = "stdin", .svfunc = &info_display_stdin, .id = 20 },
-    { .str = "stdout", .svfunc = &info_display_stdout, .id = 21 },
-    { .str = "stderr", .svfunc = &info_display_stderr, .id = 22 },
-    { .str = "logname", .svfunc = &info_display_logname, .id = 23 },
-    { .str = "logfile", .svfunc = &info_display_logfile, .id = 24 },
+    { .str = "importfile", .svfunc = &info_display_importfile, .id = 20 },
+    { .str = "stdin", .svfunc = &info_display_stdin, .id = 21 },
+    { .str = "stdout", .svfunc = &info_display_stdout, .id = 22 },
+    { .str = "stderr", .svfunc = &info_display_stderr, .id = 23 },
+    { .str = "logname", .svfunc = &info_display_logname, .id = 24 },
+    { .str = "logfile", .svfunc = &info_display_logfile, .id = 25 },
     { .str = 0, .svfunc = 0, .id = -1 }
 } ;
 
-#define MAXOPTS 26
+#define MAXOPTS 27
 #define checkopts(n) if (n >= MAXOPTS) log_die(LOG_EXIT_USER, "too many options")
 #define DELIM ','
 
@@ -215,7 +217,7 @@ static void info_get_status(resolve_service_t *res)
 
     ss_state_t sta = STATE_ZERO ;
 
-    if (res->type == TYPE_CLASSIC) {
+    if (res->type == E_PARSER_TYPE_CLASSIC) {
 
         r = s6_svc_ok(res->sa.s + res->live.scandir) ;
         if (r != 1) {
@@ -293,7 +295,7 @@ static void info_display_type(char const *field,resolve_service_t *res)
     log_flow() ;
 
     if (NOFIELD) info_display_field_name(field) ;
-    info_display_string(get_key_by_enum(list_type, res->type)) ;
+    info_display_string(enum_to_key(enum_list_parser_type, res->type)) ;
 }
 
 static void info_display_description(char const *field,resolve_service_t *res)
@@ -359,30 +361,27 @@ static void info_display_requiredby(char const *field, resolve_service_t *res)
 {
     log_flow() ;
 
+    _alloc_sa_(sa) ;
     size_t padding = 1 ;
-    int r ;
-    graph_t graph = GRAPH_ZERO ;
-    struct resolve_hash_s *hres = NULL ;
-    stralloc deps = STRALLOC_ZERO ;
+    service_graph_t graph = GRAPH_SERVICE_ZERO ;
+    uint32_t flag = GRAPH_WANT_REQUIREDBY|GRAPH_COLLECT_PARSE, nservice = 0 ;
 
     if (NOFIELD) padding = info_display_field_name(field) ;
     else { field = 0 ; padding = 0 ; }
 
-    service_graph_collect(&graph, res->sa.s + res->name, &hres, pinfo, STATE_FLAGS_TOPROPAGATE|STATE_FLAGS_WANTDOWN|STATE_FLAGS_MISSING) ;
+    if (!res->dependencies.nrequiredby)
+        goto empty ;
 
-    service_graph_compute(&graph, &hres, STATE_FLAGS_TOPROPAGATE|STATE_FLAGS_WANTDOWN|STATE_FLAGS_MISSING) ;
+    if (!sastr_clean_string(&sa, res->sa.s + res->dependencies.requiredby))
+        log_dieu(LOG_EXIT_SYS, "clean string") ;
 
-    if (!graph.mlen)
-        log_die(LOG_EXIT_USER, "services selection is not available -- please make a bug report") ;
+    if (!graph_new(&graph, (uint32_t)SS_MAX_SERVICE))
+        log_dieusys(LOG_EXIT_SYS, "allocate the service graph") ;
 
-    unsigned int list[graph.mlen] ;
+    nservice = service_graph_build_list(&graph, sa.s, sa.len, pinfo, flag) ;
 
-    int count = graph_matrix_get_requiredby(list, &graph, res->sa.s + res->name , 0) ;
-
-    if (count == -1)
-        log_dieu(LOG_EXIT_SYS,"get the requiredby list for service: ", res->sa.s + res->name) ;
-
-    if (!count) goto empty ;
+    if (!nservice && errno == EINVAL)
+        log_dieusys(LOG_EXIT_SYS, "collect resolve file of service: ", res->sa.s + res->name) ;
 
     if (GRAPH) {
 
@@ -391,125 +390,118 @@ static void info_display_requiredby(char const *field, resolve_service_t *res)
 
         depth_t d = info_graph_init() ;
 
-        if (!info_walk(&graph, res->sa.s + res->name, 0, &info_graph_display_service, 1, REVERSE, &d, padding, S_STYLE))
+        if (!service_info_walk(&graph, 0, 0, 1, REVERSE, &d, padding, S_STYLE, pinfo))
             log_dieu(LOG_EXIT_SYS,"display the requiredby list") ;
 
         goto freed ;
 
     } else {
 
-        deps.len = 0 ;
-        r = graph_matrix_get_edge_g_sorted_sa(&deps,&graph, res->sa.s + res->name, 1, 0) ;
-        if (r == -1)
-            log_dieu(LOG_EXIT_SYS, "get the requiredby list") ;
+        uint32_t pos = 0 ;
+        sa.len = 0 ;
+        FOREACH_GRAPH_SORT(service_graph_t, &graph, pos) {
+            uint32_t index = graph.g.sort[pos] ;
+            char *name = graph.g.sindex[index]->name ;
 
-        if (!r)
-            goto empty ;
+            if (!sastr_add_string(&sa, name))
+                log_die_nomem("stralloc") ;
+        }
 
         if (REVERSE)
-            if (!sastr_reverse(&deps))
+            if (!sastr_reverse(&sa))
                 log_dieu(LOG_EXIT_SYS,"reverse the selection list") ;
 
-        info_display_list(field,&deps) ;
+        info_display_list(field,&sa) ;
 
         goto freed ;
     }
 
     empty:
-        if (GRAPH)
-        {
+        if (GRAPH) {
             if (!bprintf(buffer_1,"%s\n","\\"))
                 log_dieusys(LOG_EXIT_SYS,"write to stdout") ;
             if (!bprintf(buffer_1,"%*s%s%s%s%s\n",padding, "", S_STYLE->last, log_color->warning,"None",log_color->off))
                 log_dieusys(LOG_EXIT_SYS,"write to stdout") ;
-        }
-        else
-        {
+        } else {
             info_display_empty() ;
         }
     freed:
-        graph_free_all(&graph) ;
-        hash_free(&hres) ;
-        stralloc_free(&deps) ;
+        service_graph_destroy(&graph) ;
 }
 
 static void info_display_deps(char const *field, resolve_service_t *res)
 {
     log_flow() ;
 
-    int r ;
+    _alloc_sa_(sa) ;
     size_t padding = 1 ;
-    graph_t graph = GRAPH_ZERO ;
-    struct resolve_hash_s *hres = NULL ;
-
-    stralloc deps = STRALLOC_ZERO ;
+    service_graph_t graph = GRAPH_SERVICE_ZERO ;
+    uint32_t flag = GRAPH_WANT_DEPENDS|GRAPH_COLLECT_PARSE, nservice = 0 ;
 
     if (NOFIELD) padding = info_display_field_name(field) ;
     else { field = 0 ; padding = 0 ; }
 
-    service_graph_collect(&graph, res->sa.s + res->name, &hres, pinfo, STATE_FLAGS_TOPROPAGATE|STATE_FLAGS_WANTUP|STATE_FLAGS_MISSING) ;
+    if (!res->dependencies.ndepends)
+        goto empty ;
 
-    service_graph_compute(&graph, &hres, STATE_FLAGS_TOPROPAGATE|STATE_FLAGS_WANTUP|STATE_FLAGS_MISSING) ;
+    if (!sastr_clean_string(&sa, res->sa.s + res->dependencies.depends))
+        log_dieu(LOG_EXIT_SYS, "clean string") ;
 
-    if (!graph.mlen)
-        log_die(LOG_EXIT_USER, "services selection is not available -- please make a bug report") ;
+    if (!graph_new(&graph, (uint32_t)SS_MAX_SERVICE))
+        log_dieusys(LOG_EXIT_SYS, "allocate the service graph") ;
 
-    unsigned int list[graph.mlen] ;
+    nservice = service_graph_build_list(&graph, sa.s, sa.len, pinfo, flag) ;
 
-    int count = graph_matrix_get_edge(list, &graph, res->sa.s + res->name , 0) ;
+    if (!nservice && errno == EINVAL)
+        log_dieusys(LOG_EXIT_SYS, "collect resolve file of service: ", res->sa.s + res->name) ;
 
-    if (count == -1)
-        log_dieu(LOG_EXIT_SYS,"get the dependencies list for service: ", res->sa.s + res->name) ;
+    if (GRAPH) {
 
-    if (!count) goto empty ;
-
-    if (GRAPH)
-    {
         if (!bprintf(buffer_1,"%s\n","\\"))
             log_dieusys(LOG_EXIT_SYS,"write to stdout") ;
 
         depth_t d = info_graph_init() ;
 
-        if (!info_walk(&graph, res->sa.s + res->name, 0, &info_graph_display_service, 0, REVERSE, &d, padding, S_STYLE))
+        if (!service_info_walk(&graph, 0, 0, 0, REVERSE, &d, padding, S_STYLE, pinfo))
             log_dieu(LOG_EXIT_SYS,"display the dependencies list") ;
 
         goto freed ;
-    }
-    else
-    {
-        r = graph_matrix_get_edge_g_sorted_sa(&deps,&graph, res->sa.s + res->name, 0, 0) ;
-        if (r == -1)
-            log_dieu(LOG_EXIT_SYS, "get the dependencies list") ;
 
-        if (!r)
-            goto empty ;
+    } else {
+
+        uint32_t pos = 0 ;
+        sa.len = 0 ;
+        FOREACH_GRAPH_SORT(service_graph_t, &graph, pos) {
+            uint32_t index = graph.g.sort[pos] ;
+            char *name = graph.g.sindex[index]->name ;
+
+            if (!sastr_add_string(&sa, name))
+                log_die_nomem("stralloc") ;
+        }
 
         if (REVERSE)
-            if (!sastr_reverse(&deps))
+            if (!sastr_reverse(&sa))
                 log_dieu(LOG_EXIT_SYS,"reverse the selection list") ;
 
-        info_display_list(field,&deps) ;
+        info_display_list(field,&sa) ;
 
         goto freed ;
     }
+
     empty:
-        if (GRAPH)
-        {
+        if (GRAPH) {
             if (!bprintf(buffer_1,"%s\n","\\"))
                 log_dieusys(LOG_EXIT_SYS,"write to stdout") ;
 
             if (!bprintf(buffer_1,"%*s%s%s%s%s\n",padding, "", S_STYLE->last, log_color->warning,"None",log_color->off))
                 log_dieusys(LOG_EXIT_SYS,"write to stdout") ;
-        }
-        else
-        {
+        } else {
             info_display_empty() ;
         }
 
     freed:
-        graph_free_all(&graph) ;
-        hash_free(&hres) ;
-        stralloc_free(&deps) ;
+        service_graph_destroy(&graph) ;
+
 }
 
 static void info_display_optsdeps(char const *field, resolve_service_t *res)
@@ -540,12 +532,12 @@ static void info_display_contents(char const *field, resolve_service_t *res)
 {
     log_flow() ;
 
-    size_t padding = 1 ;
-    graph_t graph = GRAPH_ZERO ;
     _alloc_sa_(sa) ;
-    struct resolve_hash_s *hres = NULL ;
+    size_t padding = 1 ;
+    service_graph_t graph = GRAPH_SERVICE_ZERO ;
+    uint32_t nservice = 0, flag = GRAPH_WANT_DEPENDS|GRAPH_WANT_REQUIREDBY ;
 
-    if (res->type != TYPE_MODULE)
+    if (res->type != E_PARSER_TYPE_MODULE)
         return ;
 
     if (NOFIELD) padding = info_display_field_name(field) ;
@@ -557,12 +549,12 @@ static void info_display_contents(char const *field, resolve_service_t *res)
     if (!sastr_clean_string(&sa, res->sa.s + res->dependencies.contents))
         log_dieu(LOG_EXIT_SYS, "clean string") ;
 
-    if (!sa.len)
-        goto empty ;
+    if (!graph_new(&graph, res->dependencies.ncontents))
+        log_dieusys(LOG_EXIT_SYS, "allocate the service graph") ;
 
-    service_graph_g(sa.s, sa.len, &graph, &hres, pinfo, STATE_FLAGS_TOPROPAGATE|STATE_FLAGS_WANTUP|STATE_FLAGS_MISSING) ;
+    nservice = service_graph_build_list(&graph, sa.s, sa.len, pinfo, flag) ;
 
-    if (!graph.mlen)
+    if (!nservice && errno == EINVAL)
         log_die(LOG_EXIT_USER, "services selection is not available -- please make a bug report") ;
 
     if (GRAPH) {
@@ -572,13 +564,24 @@ static void info_display_contents(char const *field, resolve_service_t *res)
 
         depth_t d = info_graph_init() ;
 
-        if (!info_walk(&graph, 0, 0, &info_graph_display_service, 0, REVERSE, &d, padding, S_STYLE))
+        if (!service_info_walk(&graph, 0, 0, 0, REVERSE, &d, padding, S_STYLE, pinfo))
             log_dieu(LOG_EXIT_SYS,"display the dependencies list") ;
 
         goto freed ;
 
     } else {
 
+        sa.len = 0 ;
+        uint32_t pos = 0 ;
+
+        FOREACH_GRAPH_SORT(service_graph_t, &graph, pos) {
+            uint32_t index = graph.g.sort[pos] ;
+            char *name = graph.g.sindex[index]->name ;
+
+            if (!sastr_add_string(&sa, name))
+                log_die_nomem("stralloc") ;
+        }
+
         if (REVERSE)
             if (!sastr_reverse(&sa))
                 log_dieu(LOG_EXIT_SYS,"reverse the selection list") ;
@@ -588,22 +591,18 @@ static void info_display_contents(char const *field, resolve_service_t *res)
         goto freed ;
     }
     empty:
-        if (GRAPH)
-        {
+        if (GRAPH) {
             if (!bprintf(buffer_1,"%s\n","\\"))
                 log_dieusys(LOG_EXIT_SYS,"write to stdout") ;
 
             if (!bprintf(buffer_1,"%*s%s%s%s%s\n",padding, "", S_STYLE->last, log_color->warning,"None",log_color->off))
                 log_dieusys(LOG_EXIT_SYS,"write to stdout") ;
-        }
-        else
-        {
+        } else {
             info_display_empty() ;
         }
 
     freed:
-        graph_free_all(&graph) ;
-        hash_free(&hres) ;
+        service_graph_destroy(&graph) ;
 }
 
 static void info_display_start(char const *field,resolve_service_t *res)
@@ -771,12 +770,28 @@ static void info_display_envfile(char const *field,resolve_service_t *res)
 
 }
 
+static void info_display_importfile(char const *field,resolve_service_t *res)
+{
+    log_flow() ;
+
+    if (NOFIELD) info_display_field_name(field) ;
+
+    if (res->environ.nimportfile) {
+
+        info_display_string(res->sa.s + res->environ.importfile) ;
+
+        return ;
+    }
+
+    info_display_empty() ;
+}
+
 static void info_display_logname(char const *field,resolve_service_t *res)
 {
     log_flow() ;
 
     if (NOFIELD) info_display_field_name(field) ;
-    if (res->type == TYPE_CLASSIC) {
+    if (res->type == E_PARSER_TYPE_CLASSIC) {
         if (res->logger.want) {
             info_display_string(res->sa.s + res->logger.name) ;
             return ;
@@ -792,9 +807,9 @@ static void info_display_stdin(char const *field, resolve_service_t *res)
 
     _alloc_sa_(sa) ;
     if (NOFIELD) info_display_field_name(field) ;
-    if (res->type != TYPE_MODULE && res->io.fdin.destination) {
+    if (res->type != E_PARSER_TYPE_MODULE && res->io.fdin.destination) {
 
-        if (!auto_stra(&sa, get_key_by_enum(list_io_type, res->io.fdin.type), ":", res->sa.s + res->io.fdin.destination))
+        if (!auto_stra(&sa, enum_to_key(enum_list_parser_io_type, res->io.fdin.type), ":", res->sa.s + res->io.fdin.destination))
             log_die_nomem("stralloc") ;
 
         info_display_string(sa.s) ;
@@ -810,9 +825,9 @@ static void info_display_stdout(char const *field, resolve_service_t *res)
 
     _alloc_sa_(sa) ;
     if (NOFIELD) info_display_field_name(field) ;
-    if (res->type != TYPE_MODULE && res->io.fdout.destination) {
+    if (res->type != E_PARSER_TYPE_MODULE && res->io.fdout.destination) {
 
-        if (!auto_stra(&sa, get_key_by_enum(list_io_type, res->io.fdout.type), ":", res->sa.s + res->io.fdout.destination))
+        if (!auto_stra(&sa, enum_to_key(enum_list_parser_io_type, res->io.fdout.type), ":", res->sa.s + res->io.fdout.destination))
             log_die_nomem("stralloc") ;
 
         info_display_string(sa.s) ;
@@ -828,9 +843,9 @@ static void info_display_stderr(char const *field, resolve_service_t *res)
 
     _alloc_sa_(sa) ;
     if (NOFIELD) info_display_field_name(field) ;
-    if (res->type != TYPE_MODULE && res->io.fderr.destination) {
+    if (res->type != E_PARSER_TYPE_MODULE && res->io.fderr.destination) {
 
-        if (!auto_stra(&sa, get_key_by_enum(list_io_type, res->io.fderr.type), ":", res->sa.s + res->io.fderr.destination))
+        if (!auto_stra(&sa, enum_to_key(enum_list_parser_io_type, res->io.fderr.type), ":", res->sa.s + res->io.fderr.destination))
             log_die_nomem("stralloc") ;
 
         info_display_string(sa.s) ;
@@ -845,11 +860,11 @@ static void info_display_logfile(char const *field,resolve_service_t *res)
     log_flow() ;
 
     if (NOFIELD) info_display_field_name(field) ;
-    if (res->type != TYPE_MODULE)
+    if (res->type != E_PARSER_TYPE_MODULE)
     {
-        if (res->logger.want || (res->type == TYPE_ONESHOT && res->io.fdout.destination))
+        if (res->logger.want || (res->type == E_PARSER_TYPE_ONESHOT && res->io.fdout.destination))
         {
-            if (nlog && res->io.fdout.type == IO_TYPE_S6LOG)
+            if (nlog && res->io.fdout.type == E_PARSER_IO_TYPE_S6LOG)
             {
                 stralloc log = STRALLOC_ZERO ;
                 /** the file current may not exist if the service was never started*/
@@ -873,7 +888,7 @@ static void info_display_logfile(char const *field,resolve_service_t *res)
                     if (!log.len) goto empty ;
                     log.len-- ;
                     if (!auto_stra(&log,"\n")) log_dieusys(LOG_EXIT_SYS,"append newline") ;
-                    if (log.len < 10 && res->type != TYPE_ONESHOT)
+                    if (log.len < 10 && res->type != E_PARSER_TYPE_ONESHOT)
                     {
                         if (!bprintf(buffer_1,"%s%s%s\n",log_color->warning,"None",log_color->off)) goto err ;
                     }
@@ -939,51 +954,75 @@ static void info_parse_options(char const *str,int *what)
 
 void info_status_all(void)
 {
+    log_flow() ;
+
     _alloc_sa_(sa) ;
-    struct resolve_hash_tree_s *htres = NULL ;
-    graph_t graph = GRAPH_ZERO ;
-    int flag = STATE_FLAGS_TOPROPAGATE|STATE_FLAGS_WANTUP|STATE_FLAGS_WANTDOWN|STATE_FLAGS_MISSING;
-    graph_build_tree(&graph, &htres, pinfo->base.s, E_RESOLVE_TREE_MASTER_CONTENTS) ;
+    tree_graph_t graph = GRAPH_TREE_ZERO ;
+    uint32_t f = REVERSE ? GRAPH_WANT_REQUIREDBY : GRAPH_WANT_DEPENDS ;
+    uint32_t nservice = 0 , pos = 0, flag = f|GRAPH_COLLECT_PARSE ;
 
-    if (!graph_matrix_sort_tosa(&sa, &graph))
-        log_dieu(LOG_EXIT_SYS, "get the sorted list of trees") ;
+    if (!graph_new(&graph, SS_MAX_SERVICE))
+        log_dieusys(LOG_EXIT_SYS, "allocate the graph") ;
 
-    graph_free_all(&graph) ;
+    nservice = tree_graph_build_master(&graph, pinfo, flag) ;
 
-    if (sa.len) {
+    if (!nservice && errno == EINVAL)
+        log_dieusys(LOG_EXIT_SYS, "find trees -- please make a bug report") ;
 
-        struct resolve_hash_tree_s *c, *tmp ;
-        struct resolve_hash_s *hres = NULL ;
+    _alloc_stk_(stk, graph.g.nsort * SS_MAX_TREENAME) ;
 
-        HASH_ITER(hh, htres, c, tmp) {
+    FOREACH_GRAPH_SORT(tree_graph_t, &graph, pos) {
+        uint32_t index = graph.g.sort[pos] ;
+        char *name = graph.g.sindex[index]->name ;
+        if (!stack_add_g(&stk, name)) {
+            errno = EINVAL ;
+            log_dieu(LOG_EXIT_SYS, "get the sorted list of trees") ;
+        }
+    }
 
-            if (c->tres.ncontents) {
+    if (stk.len) {
 
-                _alloc_stk_(stk, strlen(c->tres.sa.s + c->tres.contents)) ;
+        struct resolve_hash_tree_s *h ;
+        service_graph_t sg = GRAPH_SERVICE_ZERO ;
 
-                if (!stack_string_clean(&stk, c->tres.sa.s + c->tres.contents))
+        pos = 0 ;
+        FOREACH_STK(&stk, pos) {
+
+            h = hash_search_tree(&graph.hres, stk.s + pos) ;
+            if (h == NULL)
+                log_dieusys(LOG_EXIT_ZERO, "get information of tree: ", stk.s + pos, " -- please make a bug report") ;
+
+            if (h->tres.ncontents) {
+
+                _alloc_stk_(sv, strlen(h->tres.sa.s + h->tres.contents)) ;
+
+                if (!stack_string_clean(&sv, h->tres.sa.s + h->tres.contents))
                     log_dieu(LOG_EXIT_SYS, "clean string") ;
 
-                service_graph_g(stk.s, stk.len, &graph, &hres, pinfo, flag) ;
+                /** A dependencies service can be on another tree,
+                 * so used SS_MAX_SERVICE instead of stk.count. */
+                if (!graph_new(&sg, SS_MAX_SERVICE))
+                    log_dieusys(LOG_EXIT_SYS, "allocate the service graph") ;
 
-                if (!graph.mlen)
-                    log_die(LOG_EXIT_USER, "services selection is not available -- please make a bug report") ;
+                nservice = service_graph_build_list(&sg, sv.s, sv.len, pinfo, flag) ;
 
-                if (!bprintf(buffer_1,"%s%s%s%s\n","In tree: ", log_color->info, c->tres.sa.s + c->tres.name, log_color->off))
+                if (!nservice && errno == EINVAL)
+                    log_die(LOG_EXIT_USER, "build the graph -- please make a bug report") ;
+
+                if (!bprintf(buffer_1,"%s%s%s%s\n","In tree: ", log_color->info, h->tres.sa.s + h->tres.name, log_color->off))
                     log_dieusys(LOG_EXIT_SYS,"write to stdout") ;
                 if (!bprintf(buffer_1,"%s\n","\\"))
                     log_dieusys(LOG_EXIT_SYS,"write to stdout") ;
 
                 depth_t d = info_graph_init() ;
 
-                if (!info_walk(&graph, 0, c->tres.sa.s + c->tres.name, &info_graph_display_service, 0, REVERSE, &d, 0, S_STYLE))
+                if (!service_info_walk(&sg, 0, h->tres.sa.s + h->tres.name, 0, REVERSE, &d, 0, S_STYLE, pinfo))
                     log_dieu(LOG_EXIT_SYS,"display the dependencies list") ;
 
                 if (buffer_puts(buffer_1,"\n") == -1)
                     log_dieusys(LOG_EXIT_SYS,"write to stdout") ;
 
-                graph_free_all(&graph) ;
-                hash_free(&hres) ;
+                service_graph_destroy(&sg) ;
             }
         }
 
@@ -991,7 +1030,7 @@ void info_status_all(void)
         log_dieusys(LOG_EXIT_SYS, "find trees -- please make a bug report") ;
     }
 
-    hash_free_tree(&htres) ;
+    tree_graph_destroy(&graph) ;
 }
 
 void info_status_one(const char *service, int *what)
@@ -1051,6 +1090,7 @@ int ssexec_status(int argc, char const *const *argv, ssexec_t *info)
         "Stop script",
         "Environment source",
         "Environment file",
+        "Environment ImportFile",
         "StdIn",
         "StdOut",
         "StdErr",
@@ -1107,5 +1147,4 @@ int ssexec_status(int argc, char const *const *argv, ssexec_t *info)
     }
 
     return 0 ;
-
 }
